@@ -20,13 +20,10 @@
 #include "gretl.h"
 #include <errno.h>
 #include <ctype.h>
-#ifndef G_OS_WIN32
-# include <gtkextra/gtksheet.h>
-# include <gtkextra/gtkitementry.h>
-#else
-# include <gtksheet.h>
-# include <gtkitementry.h>
-#endif
+#include <gtkextra/gtksheet.h>
+#include <gtkextra/gtkitementry.h>
+
+#include "ssheet.h"
 
 #define DEFAULT_PRECISION 6
 #define DEFAULT_SPACE 8
@@ -39,6 +36,7 @@ static GtkWidget *sheet_popup;
 
 static char newvarname[9], newobsmarker[9];
 static int numcolumns, numrows;
+static int sheet_modified;
 
 static void sheet_add_var (void);
 static void sheet_add_obs (void);
@@ -47,48 +45,18 @@ static void sheet_clear (gpointer *data, guint all, GtkWidget *w);
 static void get_data_from_sheet (void);
 
 static GtkItemFactoryEntry sheet_items[] = {
-    { _("/_Observation"), NULL, NULL, 0, "<Branch>" },
-    { _("/Observation/_Append obs"), NULL, sheet_add_obs, 0, NULL },
-    { _("/Observation/_Insert obs"), NULL, sheet_insert_obs, 0, NULL },
-    { _("/_Variable"), NULL, NULL, 0, "<Branch>" },
-    { _("/Variable/_Add"), NULL, sheet_add_var, 0, NULL },
-    { _("/_Clear"), NULL, NULL, 0, "<Branch>" },
-    { _("/_Clear/_Selected cells"), NULL, sheet_clear, 0, NULL },
-    { _("/_Clear/_All data"), NULL, sheet_clear, 1, NULL }
+    { N_("/_Observation"), NULL, NULL, 0, "<Branch>" },
+    { N_("/Observation/_Append obs"), NULL, sheet_add_obs, 0, NULL },
+    { N_("/Observation/_Insert obs"), NULL, sheet_insert_obs, 0, NULL },
+    { N_("/_Variable"), NULL, NULL, 0, "<Branch>" },
+    { N_("/Variable/_Add"), NULL, sheet_add_var, 0, NULL },
+    { N_("/_Clear"), NULL, NULL, 0, "<Branch>" },
+    { N_("/_Clear/_Selected cells"), NULL, sheet_clear, 0, NULL },
+    { N_("/_Clear/_All data"), NULL, sheet_clear, 1, NULL }
 };
 
 /* ........................................................... */
 
-static int check_atof (char *numstr)
-{
-    char *test;
-    extern int errno;
-
-    (void) strtod(numstr, &test);
-
-    if (strcmp(numstr, test) == 0) {
-	sprintf(errtext, _("'%s' -- no numeric conversion performed!"), numstr);
-	errbox(errtext);
-	return 1;
-    }
-    if (test[0] != '\0') {
-	if (isprint(test[0]))
-	    sprintf(errtext, _("Extraneous character '%c' in data"), test[0]);
-	else
-	    sprintf(errtext, _("Extraneous character (0x%x) in data"), test[0]);
-	errbox(errtext);
-	return 1;
-    }
-    if (errno == ERANGE) {
-	sprintf(errtext, _("'%s' -- number out of range!"), numstr);
-	errbox(errtext);
-	return 1;
-    }
-    return 0;
-}
-
-/* ........................................................... */
-
 static int check_data_in_sheet (void)
 {
     gint err, i, t, n = datainfo->n;
@@ -101,7 +69,10 @@ static int check_data_in_sheet (void)
 	    if (celltext != NULL) {
 		strncpy(numstr, celltext, 31);
 		err = check_atof(numstr);
-		if (err) return 1;
+		if (err) {
+		    errbox(get_gretl_errmsg());
+		    return 1;
+		}
 	    }
 	}
     }
@@ -112,23 +83,29 @@ static int check_data_in_sheet (void)
 
 static void name_new_var (GtkWidget *widget, dialog_t *ddata) 
 {
-    char *edttext;
+    char *buf = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
 
-    edttext = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
-    if (*edttext == '\0' || validate_varname(edttext)) return;
-    strcpy(newvarname, edttext);
+    if (blank_entry(buf, ddata)) return;
+    if (validate_varname(buf)) return;
+
+    *newvarname = '\0';
+    strncat(newvarname, buf, 8);
+
+    close_dialog(ddata);
 }
 
 /* ........................................................... */
 
 static void name_new_obs (GtkWidget *widget, dialog_t *ddata) 
 {
-    char *edttext;
+    char *buf = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
 
-    edttext = gtk_entry_get_text (GTK_ENTRY (ddata->edit));
-    if (*edttext == '\0') return;
-    strncpy(newobsmarker, edttext, 8);
-    newobsmarker[8] = '\0';
+    if (blank_entry(buf, ddata)) return;
+
+    *newobsmarker = '\0';
+    strncat(newobsmarker, buf, 8);
+
+    close_dialog(ddata);
 }
 
 /* ........................................................... */
@@ -138,9 +115,9 @@ static void name_var_dialog (void)
     edit_dialog (_("gretl: name variable"), 
 		 _("Enter name for new variable\n"
 		 "(max. 8 characters)"),
-		 NULL, 1,
-		 _("Apply"), name_new_var, mdata, 
-		 _("Cancel"), NULL, NULL, 0, 0);
+		 NULL, 
+		 name_new_var, mdata, 
+		 0, 0);
 }
 
 /* ........................................................... */
@@ -150,9 +127,9 @@ static void new_case_dialog (void)
     edit_dialog (_("gretl: case marker"), 
 		 _("Enter case marker for new obs\n"
 		 "(max. 8 characters)"),
-		 NULL, 1,
-		 _("Apply"), name_new_obs, mdata, 
-		 _("Cancel"), NULL, NULL, 0, 0);
+		 NULL, 
+		 name_new_obs, mdata, 
+		 0, 0);
 }
 
 /* ........................................................... */
@@ -161,8 +138,10 @@ static void sheet_add_var (void)
 {
     GtkSheet *sheet = GTK_SHEET(gretlsheet); 
 
-    newvarname[0] = '\0';
+    *newvarname = '\0';
+
     name_var_dialog();
+
     if (*newvarname != '\0') {
 	gtk_sheet_add_column(sheet, 1);
 	numcolumns++;
@@ -170,6 +149,7 @@ static void sheet_add_var (void)
 	gtk_sheet_set_column_title(sheet, numcolumns-1, newvarname);
 	gtk_sheet_set_active_cell(sheet, 0, numcolumns-1);
 	gtk_sheet_moveto(sheet, 0, numcolumns-1, 0.0, 0.8);
+	sheet_modified = 1;
     }
 }
 
@@ -178,21 +158,21 @@ static void sheet_add_var (void)
 static void sheet_add_obs (void)
 {
     GtkSheet *sheet = GTK_SHEET(gretlsheet);
-    char rowlabel[10];
+    char rowlabel[OBSLEN];
+
+    *newobsmarker = '\0';
 
-    newobsmarker[0] = '\0';
     if (datainfo->markers) new_case_dialog();
+
     if (!datainfo->markers || *newobsmarker != '\0') {
 	gtk_sheet_add_row(sheet, 1);
 	numrows++;
-	if (datainfo->markers) 
-	    strcpy(rowlabel, newobsmarker);
-	else 
-	    ntodate(rowlabel, numrows-1, datainfo);
+	get_full_obs_string(rowlabel, numrows-1, datainfo);
 	gtk_sheet_row_button_add_label(sheet, numrows-1, rowlabel);
 	gtk_sheet_set_row_title(sheet, numrows-1, rowlabel);
 	gtk_sheet_set_active_cell(sheet, numrows-1, 0);
 	gtk_sheet_moveto(sheet, numrows-1, 0, 0.8, 0.0);
+	sheet_modified = 1;
     }
 }
 
@@ -205,7 +185,9 @@ static void sheet_insert_obs (void)
     int i;
 
     newobsmarker[0] = '\0';
+
     if (datainfo->markers) new_case_dialog();
+
     if (!datainfo->markers || *newobsmarker != '\0') {
 	gtk_sheet_insert_rows(sheet, sheet->active_cell.row, 1);
 	numrows++;
@@ -217,11 +199,12 @@ static void sheet_insert_obs (void)
 				    rowlabel);
 	} else {
 	    for (i=sheet->active_cell.row; i<numrows; i++) {
-		ntodate(rowlabel, i, datainfo);
+		get_full_obs_string(rowlabel, i, datainfo);
 		gtk_sheet_row_button_add_label(sheet, i, rowlabel);
 		gtk_sheet_set_row_title(sheet, i, rowlabel);
 	    }
 	}
+	sheet_modified = 1;
     }
 }
 
@@ -264,25 +247,25 @@ static gint popup_activated (GtkWidget *widget, gpointer data)
 static GtkWidget *build_menu (GtkWidget *sheet)
 {
     static char *items[]={
-        _("Add Variable"),
-        _("Add Observation"),
-        _("Insert Observation"),
-        _("Clear Cells")
+        N_("Add Variable"),
+        N_("Add Observation"),
+        N_("Insert Observation"),
+        N_("Clear Cells")
     };
     GtkWidget *menu;
     GtkWidget *item;
-    int i;
+    int i, n_items = sizeof items / sizeof items[0];
 
     menu = gtk_menu_new();
 
-    for (i=0; i<(sizeof items / sizeof items[0]); i++) {
-        item = gtk_menu_item_new_with_label(items[i]);
+    for (i=0; i<n_items; i++) {
+        item = gtk_menu_item_new_with_label(_(items[i]));
         gtk_signal_connect(GTK_OBJECT(item), "activate",
                            (GtkSignalFunc) popup_activated,
-                           items[i]);
+                           _(items[i]));
         GTK_WIDGET_SET_FLAGS (item, GTK_SENSITIVE | GTK_CAN_FOCUS);
         gtk_widget_show(item);
-        gtk_menu_append(GTK_MENU(menu),item);
+        gtk_menu_append(GTK_MENU(menu), item);
     }
     return menu;
 }
@@ -315,12 +298,18 @@ static void parse_numbers (GtkWidget *widget, gpointer data)
     gchar *entrytext;
     char label[32];
 
-    /* FIXME error checking here? */
     entrytext = gtk_entry_get_text(GTK_ENTRY(sheet->sheet_entry));
     if (entrytext == NULL) return;
-    sprintf(label, "%.*f", DEFAULT_PRECISION, atof(entrytext));
+
+    if (check_atof(entrytext)) {
+	errbox(get_gretl_errmsg());
+	*label = 0;
+    } else {
+	sprintf(label, "%.*f", DEFAULT_PRECISION, atof(entrytext));
+    }
     gtk_sheet_set_cell(sheet, sheet->active_cell.row,
                        sheet->active_cell.col, GTK_JUSTIFY_RIGHT, label); 
+    sheet_modified = 1;
 }
 
 /* ........................................................... */
@@ -407,20 +396,24 @@ static gint activate_sheet_cell (GtkWidget *w, gint row, gint column,
 
     sheet_entry = GTK_ENTRY(gtk_sheet_get_entry(sheet));
 
-    if (sheet->column[column].name && sheet->row[row].name)
+    if (sheet->column[column].name && sheet->row[row].name) {
         sprintf(cell,"  %s:%s  ", 
                 sheet->column[column].name, 
                 sheet->row[row].name);
-    else 
+    } else {
         sprintf(cell,"  %s:%d  ", sheet->column[column].name, row);
+    }
 
     gtk_label_set(GTK_LABEL(locator), cell);
     gtk_entry_set_max_length(GTK_ENTRY(topentry),
                              GTK_ENTRY(sheet_entry)->text_max_length);
-    if ((text = gtk_entry_get_text(GTK_ENTRY(gtk_sheet_get_entry(sheet)))))
+
+    if ((text = gtk_entry_get_text(GTK_ENTRY(gtk_sheet_get_entry(sheet))))) {
         gtk_entry_set_text(GTK_ENTRY(topentry), text);
-    else
+    } else {
         gtk_entry_set_text(GTK_ENTRY(topentry), "");
+    }
+
     return TRUE;
 }
 
@@ -444,7 +437,6 @@ static void get_data_from_sheet (void)
 	n = datainfo->n;
     }
     if (newvars > 0) {
-	/*  g_print("%d vars added\n", newvars); */
 	if (dataset_add_vars(newvars, &Z, datainfo)) {
 	    errbox(_("Failed to allocate memory for new data"));
 	    return;
@@ -452,11 +444,12 @@ static void get_data_from_sheet (void)
 	for (i=0; i<newvars; i++) { 
 	    strcpy(datainfo->varname[i+oldv], 
 		   sheet->column[i+oldv-1].button.label);
-	    strcpy(datainfo->label[i+oldv], "");
+	    strcpy(VARLABEL(datainfo, i + oldv), "");
 	}
     }
     for (i=0; i<datainfo->v-1; i++) {
 	for (t=0; t<n; t++) {
+	    if (datainfo->vector[i+1] == 0 && t > 0) continue;
 	    celltext = gtk_sheet_cell_get_text(sheet, t, i);
 	    if (celltext != NULL)
 		Z[i+1][t] = atof(celltext);
@@ -471,11 +464,13 @@ static void get_data_from_sheet (void)
 	    strcpy(datainfo->S[t], sheet->row[t].button.label); 
     }
     data_status |= (GUI_DATA|MODIFIED_DATA);
-    register_data(NULL, 0);
+    register_data(NULL, NULL, 0);
     if (missobs)
 	infobox(_("Warning: there were missing observations"));
     else
 	infobox(_("Data updated OK"));
+
+    sheet_modified = 0;
 }
 
 /* ........................................................... */
@@ -494,10 +489,7 @@ static void add_data_to_sheet (GtkWidget *w)
     }
     numcolumns = i;
     for (t=0; t<n; t++) {
-	if (datainfo->markers) 
-	    strcpy(rowlabel, datainfo->S[t]);
-	else
-	    ntodate(rowlabel, t, datainfo);
+	get_full_obs_string(rowlabel, t, datainfo);
 	gtk_sheet_row_button_add_label(sheet, t, rowlabel);
 	gtk_sheet_set_row_title(sheet, t, rowlabel);
     }
@@ -505,6 +497,7 @@ static void add_data_to_sheet (GtkWidget *w)
     /* enter the data values */
     for (t=0; t<n; t++) {
 	for (i=0; i<datainfo->v-1; i++) {
+	    if (datainfo->vector[i+1] == 0 && t > 0) continue;
 	    xx = Z[i+1][t];
 	    if (!na(xx)) 
 		sprintf(label, "%.*f", DEFAULT_PRECISION, Z[i+1][t]);
@@ -515,6 +508,8 @@ static void add_data_to_sheet (GtkWidget *w)
     }
     gtk_signal_connect(GTK_OBJECT(sheet), "key_press_event",
 		       (GtkSignalFunc) clipboard_handler, NULL);
+
+    sheet_modified = 0;
 }
 
 /* ........................................................... */
@@ -531,7 +526,7 @@ static void add_skel_to_sheet (GtkWidget *w)
     }
     numcolumns = i;
     for (t=0; t<n; t++) {
-	ntodate(rowlabel, t, datainfo);
+	get_full_obs_string(rowlabel, t, datainfo);
 	gtk_sheet_row_button_add_label(sheet, t, rowlabel);
 	gtk_sheet_set_row_title(sheet, t, rowlabel);
     }
@@ -539,6 +534,8 @@ static void add_skel_to_sheet (GtkWidget *w)
     for (i=0; i<datainfo->v-1; i++) 
 	for (t=0; t<n; t++) 
 	    gtk_sheet_set_cell(sheet, t, i, GTK_JUSTIFY_RIGHT, ""); 
+
+    sheet_modified = 0;
 }
 
 /* ........................................................... */
@@ -550,6 +547,25 @@ static void free_spreadsheet (GtkWidget *widget, gpointer data)
 
 /* ........................................................... */
 
+static void maybe_exit_sheet (GtkWidget *w, GtkWidget *swin)
+{
+    int resp;
+
+    if (sheet_modified) {
+	resp = yes_no_dialog ("gretl", 
+			      _("Do you want to save changes you have\n"
+				"made to the current data set?"), 1);
+	if (resp == GRETL_YES) {
+	    get_data_from_sheet();
+	}
+	else if (resp == GRETL_CANCEL || resp == -1) return;
+    }
+  
+    gtk_widget_destroy(swin);
+}
+
+/* ........................................................... */
+
 void show_spreadsheet (DATAINFO *pdinfo) 
 {
     GtkWidget *tmp, *button_box;
@@ -581,6 +597,9 @@ void show_spreadsheet (DATAINFO *pdinfo)
     accel = gtk_accel_group_new();
     ifac = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", 
 				accel);
+#ifdef ENABLE_NLS
+    gtk_item_factory_set_translate_func(ifac, menu_translate, NULL, NULL);
+#endif
     gtk_item_factory_create_items(ifac, 
 				  sizeof sheet_items / sizeof sheet_items[0],
 				  sheet_items, sheetwin);
@@ -629,7 +648,7 @@ void show_spreadsheet (DATAINFO *pdinfo)
     tmp = gtk_button_new_with_label(_("Close"));
     gtk_box_pack_start (GTK_BOX (button_box), tmp, FALSE, TRUE, 0);
     gtk_signal_connect(GTK_OBJECT(tmp), "clicked",
-		       GTK_SIGNAL_FUNC(delete_widget), sheetwin);
+		       GTK_SIGNAL_FUNC(maybe_exit_sheet), sheetwin);
     gtk_widget_show(tmp);
 
     gtk_signal_connect(GTK_OBJECT(gtk_sheet_get_entry(GTK_SHEET(gretlsheet))),
@@ -653,10 +672,11 @@ void show_spreadsheet (DATAINFO *pdinfo)
 
     GTK_SHEET_SET_FLAGS(gretlsheet, GTK_SHEET_AUTORESIZE);
 
-    if (pdinfo != NULL)
+    if (pdinfo != NULL) {
 	add_skel_to_sheet(gretlsheet);
-    else 
+    } else {
 	add_data_to_sheet(gretlsheet);
+    }
 
     gtk_widget_show(sheetwin);
 }
