@@ -20,26 +20,38 @@
 /* compare.h for gretl */
 
 #include <stdio.h>
+#include "gretl_matrix.h"
 
-typedef enum {
-    NONE,      /* not an auxiliary regression */
+enum series_save_opts {
+    SAVE_LEVERAGE  = 1 << 0,
+    SAVE_INFLUENCE = 1 << 1,
+    SAVE_DFFITS    = 1 << 2,
+    SAVE_TREND     = 1 << 3,
+    SAVE_CYCLE     = 1 << 4
+};
+
+enum aux_codes {
+    AUX_NONE,  /* not an auxiliary regression */
     AUX_SQ,    /* aux. regression for nonlinearity (squared terms) */
     AUX_LOG,   /* aux. regression for nonlinearity (log terms) */
     AUX_CHOW,  /* aux. regression for Chow test */
     AUX_ADD,   /* aux. regression for adding variables */
     AUX_AR,    /* aux. regression for autocorrelation test */
+    AUX_SCR,   /* regression showing serial correlation-robust std errs */
     AUX_WHITE, /* aux. regression for heteroskedasticity (White's test) */
     AUX_COINT, /* aux. regression for cointegreation test */
     AUX_ARCH,  /* aux. regression for ARCH test */
     AUX_ADF,   /* aux. regression for augmented Dickey-Fuller test */
-    AUX_OMIT   /* aux. regression for omitting variables */
-} aux_codes;
+    AUX_OMIT,  /* aux. regression for omitting variables */
+    AUX_RESET, /* aux. regression for Ramsey's RESET */
+    AUX_SYS,   /* single equation from multivariate system */
+    AUX_AUX    /* auxiliary regression not otherwise specified */
+};
 
 typedef struct {
     int m1;        /* ID for first model */
     int m2;        /* ID for second model */
-    int ols;       /* was the first model estimated via OLS? */
-    int discrete;  /* logit or probit model? */
+    int ci;        /* estimator code for the first */
     int dfn;       /* numerator degrees of freedom */
     int dfd;       /* denominator degrees of freedom */ 
     double F;      /* F test statistic */
@@ -53,15 +65,18 @@ typedef struct {
 int auxreg (LIST addvars, 
 	    MODEL *orig, MODEL *new, int *model_count, 
 	    double ***pZ, DATAINFO *pdinfo, 
-	    const int aux_code, 
-	    PRN *prn, GRETLTEST *test);
+	    int aux_code, 
+	    PRN *prn, GRETLTEST *test, unsigned long opt);
 
 int omit_test (LIST omitvars, MODEL *orig, MODEL *new, 
 	       int *model_count, 
 	       double ***pZ, DATAINFO *pdinfo, 
-	       PRN *prn);
+	       PRN *prn, unsigned long opt);
 
-int autocorr_test (MODEL *pmod, 
+int reset_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo, 
+		PRN *prn, GRETLTEST *test);
+
+int autocorr_test (MODEL *pmod, int order,
 		   double ***pZ, DATAINFO *pdinfo, 
 		   PRN *prn, GRETLTEST *test);
 
@@ -72,9 +87,26 @@ int chow_test (const char *line, MODEL *pmod,
 int cusum_test (MODEL *pmod, 
 		double ***pZ, DATAINFO *pdinfo, 
 		PRN *prn, 
-		const PATHS *ppaths, 
+		PATHS *ppaths, 
 		GRETLTEST *test);
 
 int hausman_test (MODEL *pmod, 
 		  double ***pZ, DATAINFO *pdinfo, 
-		  const PATHS *ppaths, PRN *prn);
+		  PRN *prn);
+
+int leverage_test (MODEL *pmod, 
+		   double ***pZ, DATAINFO *pdinfo, 
+		   PRN *prn, PATHS *ppaths, 
+		   unsigned long oflag);
+
+int add_leverage_values_to_dataset (double ***pZ, DATAINFO *pdinfo,
+				    gretl_matrix *m, unsigned char opt);
+
+int mp_ols (const LIST list, const char *pos,
+	    double ***pZ, DATAINFO *pdinfo, 
+	    PRN *prn); 
+
+int sum_test (LIST sumvars, MODEL *pmod, 
+	      double ***pZ, DATAINFO *pdinfo, 
+	      PRN *prn);
+
