@@ -20,7 +20,7 @@
 #include <stdio.h>
 #include <string.h>
 
-typedef enum {
+enum data_options {
     GRETL_DATA_FLOAT = 1, /* single-precision binary data */
     GRETL_DATA_DOUBLE,    /* double-precision binary data */
     GRETL_DATA_OCTAVE,    /* data in Gnu Octave format */
@@ -29,9 +29,9 @@ typedef enum {
     GRETL_DATA_R_ALT,     /* data in alternate Gnu R format */
     GRETL_DATA_GZIPPED,   /* gzipped data */
     GRETL_DATA_TRAD       /* traditional (ESL-style) data */
-} data_options;
+};
 
-typedef enum {
+enum gretl_filetypes {
     GRETL_NATIVE_DATA,    /* gretl native format data file */
     GRETL_XML_DATA,       /* gretl xml format data file */
     GRETL_CSV_DATA,       /* comma-separated data file */
@@ -39,15 +39,25 @@ typedef enum {
     GRETL_GNUMERIC,       /* gnumeric workbook data */
     GRETL_EXCEL,          /* MS Excel spreadsheet data */
     GRETL_SCRIPT,         /* file containing gretl commands */
+    GRETL_NATIVE_DB,      /* gretl database */
+    GRETL_RATS_DB,        /* RATS 4.0 database */
     GRETL_UNRECOGNIZED    /* none of the above */
-} gretl_filetypes;
+};
 
-typedef enum {
+enum clear_codes {
     CLEAR_FULL,
     CLEAR_SUBSAMPLE
-} clear_codes;
+};
 
-#define free_datainfo(p) clear_datainfo(p, 0); free(p);
+enum data_open_codes {
+    DATA_NONE,
+    DATA_CLEAR,
+    DATA_APPEND
+};
+
+
+#define free_datainfo(p) do { if (p != NULL) { clear_datainfo(p, 0); free(p); } \
+                            } while (0);
 
 /* functions follow */
 
@@ -67,16 +77,22 @@ int start_new_Z (double ***pZ, DATAINFO *pdinfo, int resample);
 
 int dateton (const char *date, const DATAINFO *pdinfo);
 
-void ntodate (char *datestr, const int nt, const DATAINFO *pdinfo);
+char *ntodate (char *datestr, int nt, const DATAINFO *pdinfo);
+
+char *ntodate_full (char *datestr, int t, const DATAINFO *pdinfo);
 
 int get_info (const char *hdrfile, PRN *prn);
 
-int get_precision (double *x, int n);
+int get_precision (double *x, int n, int placemax);
+
+double get_date_x (int pd, const char *obs);
 
 int write_data (const char *fname, const int *list, 
 		double **Z, const DATAINFO *pdinfo, 
 	        int opt, PATHS *ppaths);
 
+int data_report (const DATAINFO *pdinfo, PATHS *ppaths, PRN *prn);
+
 int is_gzipped (const char *fname);
 
 int has_gz_suffix (const char *fname);
@@ -85,34 +101,31 @@ void gz_switch_ext (char *targ, char *src, char *ext);
 
 int merge_data (double ***pZ, DATAINFO *pdinfo,
 		double **addZ, DATAINFO *addinfo,
-		PRN *prn, int gui);
+		PRN *prn);
 
-int get_data (double ***pZ, DATAINFO *pdinfo, 
-	      char *datfile, PATHS *ppaths, 
-	      const int data_status, PRN *prn);
+int gretl_get_data (double ***pZ, DATAINFO **ppdinfo, 
+		    char *datfile, PATHS *ppaths, 
+		    int data_status, PRN *prn);
 
 int open_nulldata (double ***pZ, DATAINFO *pdinfo, 
-		   const int data_status, const int length,
+		   int data_status, int length,
 		   PRN *prn);
 
-int import_csv (double ***pZ, DATAINFO *pdinfo, 
-                const char *fname, PRN *prn);
+int import_csv (double ***pZ, DATAINFO **ppdinfo, 
+                const char *fname, PATHS *ppaths, PRN *prn);
 
-int import_box (double ***pZ, DATAINFO *pdinfo, 
+int import_box (double ***pZ, DATAINFO **ppdinfo, 
 		const char *fname, PRN *prn);
 
 int add_case_markers (DATAINFO *pdinfo, const char *fname);
 
 int detect_filetype (char *fname, PATHS *ppaths, PRN *prn);
 
-int get_xmldata (double ***pZ, DATAINFO *pdinfo, char *fname,
+int check_varname (const char *varname);
+
+int get_xmldata (double ***pZ, DATAINFO **ppdinfo, char *fname,
 		 PATHS *ppaths, int data_status, PRN *prn, int gui); 
 
 char *get_xml_description (const char *fname);
 
-int open_plugin (const PATHS *ppaths, const char *plugin, void **handle);
-
-void *get_plugin_function (const char *funcname, void *handle);
-
-void close_plugin (void *handle);
-
+int check_atof (const char *numstr);
