@@ -19,7 +19,9 @@
 
 /* functions follow */
 
-MODEL logit_probit (LIST list, 
+MODEL logit_probit (int *list, 
 		    double ***pZ, DATAINFO *pdinfo, 
 		    int opt);
 
+
+
