@@ -22,15 +22,22 @@
 #ifndef INTERACT_H
 #define INTERACT_H
 
-typedef struct {
-    char cmd[9];
-    char str[4];
-    int ci;
-    int nolist;
-    int *list;
-    char *param;
-    int errcode;
-} CMD;
+#define MAXSAVENAME 32
+#define VARDUP     999
+
+typedef struct _CMD CMD;
+
+struct _CMD {
+    char cmd[9];                /* command word */
+    int ci;                     /* command index number */
+    unsigned long opt;          /* option flags */
+    char savename[MAXSAVENAME]; /* name used to save an object from the command */
+    char str[4];                /* used, e.g., in "multiply" command */
+    int nolist;                 /* = 1 if the command does not take a list */
+    int *list;                  /* list of variables by ID number */
+    char *param;                /* general-purpose parameter to command */
+    int errcode;                /* error code */
+};
 
 enum option_codes {
     OPT_BATCH = 1,
@@ -54,21 +61,25 @@ int help (const char *cmd, const char *helpfile, PRN *prn);
 int fcast (const char *line, const MODEL *pmod, DATAINFO *pdinfo, 
 	   double ***pZ);
 
-int add_new_var (DATAINFO *pdinfo, double ***pZ, GENERATE *genr);
-
-int parseopt (const char *s);
+int parseopt (const char **argv, int argc, char *fname, int *english);
 
 int shell (const char *arg);
 
 void echo_cmd (CMD *pcmd, const DATAINFO *pdinfo, const char *line, 
-	       const int batch, const int gui, const int oflag, 
-	       PRN *prn);
+	       int batch, int gui, PRN *prn);
 
 int simple_commands (CMD *cmd, const char *line, 
 		     double ***pZ, DATAINFO *datainfo, PATHS *paths,
-		     const int batch, const int oflag, 
+		     int batch, PRN *prn);
+
+int call_pca_plugin (CORRMAT *corrmat, double ***pZ,
+		     DATAINFO *pdinfo, unsigned long *pflag,
 		     PRN *prn);
 
+int ready_for_command (const char *line);
+
+void safe_print_line (const char *line, PRN *prn);
+
 #endif /* INTERACT_H */
 
 
