@@ -22,36 +22,31 @@
 */  
 
 #include "libgretl.h" 
+#include "../../cephes/libprob.h"
  
-extern void _putxx (const double xx);
+static void putxx (double xx);
+static void pnormal (void);
+static void ptvalue (void);
+static void pchisq (void);
+static void pfvalue (void);
+static void pgamma (void);
+static double getx (void);
+static void getdf (const char *str);
 
-static void _pnormal (void);
-static void _ptvalue (void);
-static void _pchisq (void);
-static void _pfvalue (void);
-static void _pgamma (void);
-static double _getvalue (void);
-static void _enterdf (const char *str);
-
-double _gammadist (double s1, double s2, double x, int control);
-
-const char negval[] = _("\nEnter x value (value < 0 will exit menu): "); 
+const char negval[] = N_("\nEnter x value (value < 0 will exit menu): "); 
 
 /**
- * _tcrit95:
+ * tcrit95:
  * @df: degrees of freedom.
  * 
  * Returns: the 95 percent critical value for the t distribution
- * with @df degrees of freedom.
+ * with @df degrees of freedom (two-sided)
  *
  */
 
-double _tcrit95 (const int df)
+double tcrit95 (int df)
 {
-    double x = 1.960;
-    
-    while (tprob(x, df) > .05) x += .001;
-    return x;
+    return stdtri(df, 0.975);
 }
 
 /**
@@ -63,11 +58,10 @@ double _tcrit95 (const int df)
  *
  */
 
-double rhocrit95 (const int n)
+double rhocrit95 (int n)
 {
-    double x = 1.960;
+    double x = stdtri(n - 2, 0.975);
     
-    while (tprob(x, n - 2) > .05) x += .001;
     return sqrt(x*x / (x*x - 2 + n));
 }
 
@@ -80,13 +74,14 @@ double rhocrit95 (const int n)
  *
  */
 
-double tprob (const double x, const int df)
+double tprob (double x, int df)
 {
     double xx;
 
     if (df <= 0) return -1.0;
     xx = x*x;
     return fdist(xx, 1, df);
+    /* return 1.0 - stdtr(df, x); */
 }
 
 /**
@@ -100,63 +95,10 @@ double tprob (const double x, const int df)
  * negative.
  */
 
-double fdist (const double x, const int dfn, const int dfd)
+double fdist (double x, int dfn, int dfd)
 {
-    int ia, ib, i, j;
-    double xi, fia, fi1, fi2, w, zz, p, zy, d, zk;
-
-    if (dfn <= 0 || dfd <= 0) return -1;
-    if (x <= 0) return 1.0;
-    ia = 2*(dfn/2)-dfn+2;
-    ib = 2*(dfd/2)-dfd+2;
-    w = x*dfn/dfd;
-    zz = 1./(1.+w);
-    if (ia == 1) {
-	if(ib == 1) {
-	    p = sqrt(w);
-	    zy = .3183098862;
-	    d = zy*zz/p;
-	    p = 2.0*zy*atan(p);
-	} else {
-	    p = sqrt(w*zz);
-	    d = .5*p*zz/w;
-	}
-    } else {
-	if(ib == 1) {
-	    p = sqrt(zz);
-	    d = .5*zz*p;
-	    p = 1.0-p;
-	} else {
-	    d = zz*zz;
-	    p = w*zz;
-	}
-    }
-    zy = 2.*w/zz;
-    if (ia == 1) {
-	fia = (double) ia;
-	for (i=ib+2; i<=dfd; i=i+2) {
-	    fi1 = (double) (i-1);
-	    fi2 = (double) (i-2);
-	    d = (1.0+fia/fi2)*d*zz;
-	    p = p+d*zy/fi1;
-	}
-    } else {
-	xi = (double) ((dfd - 1)/2);
-	zk = pow(zz,xi);
-	d = d*zk*dfd/ib;
-	p = p*zk+w*zz*(zk-1.0)/(zz-1.0);
-    }
-    zy = w*zz;
-    zz = 2.0/zz;
-    ib = dfd-2;
-    for (i=ia+2; i<=dfn; i=i+2) {
-	j = i+ib;
-	d = zy*d*j/(i - 2);
-	p = p - zz*d/j;
-    }
-    if (p <= 0.0) p = 0.0;
-    if (p >= 1.0) p = 1.0;
-    return (1.0 - p);
+    if (x < 0) return 1.0;
+    else return fdtrc(dfn, dfd, x);
 }
 
 /**
@@ -168,64 +110,10 @@ double fdist (const double x, const int dfn, const int dfd)
  * Chi-squared(@df) is greater than @x.
  */
 
-double chisq (const double x, const int df)
+double chisq (double x, int df)
 {
-    double aa, bb, absx, p, zy, zz, h, d, sum, xs, xh, xi, xx, x2;
-    int i, m, iy;
-
-    if (x <= 0.0 || df <= 0) return 1.0;
-    if (df == 1) {
-	zy = sqrt(x);
-	p = 2.0 * normal(zy);
-	return p;
-    }
-    h = (double) df;
-    if (df <= 30) {
-	x2 = -x/2.0;
-	zy = h/2.0;
-	iy = (int) zy;
-	zy = zy - iy;
-	if (zy > 0.01) {
-	    aa = (h - .999)/2.0;
-	    m = (int) aa;
-	    d = 1.0;
-	    zy = sqrt(x);
-	    sum = 0.0;
-	    for (i=1; i<=m; i++) {
-		xi = (double) i;
-		d = d*(2.0*xi-1.0);
-		sum += (pow(x, xi) / (zy*d));
-	    }
-	    xs = normal(zy);
-	    p = 2.*xs + 0.7978845612587234 * sum * exp(x2);
-	} else {
-	    aa = (h - 1.999)/2.0;
-	    m = (int) aa; 
-	    d = 1.0;
-	    sum = 0.0;
-	    for (i=1; i<=m; i++) {
-		xi = (double) i;
-		d *= 2.0*i;
-		sum += pow(x, xi) / d;
-	    }
-	    if (x <= 175.0) p = (1.0 + sum) * exp(x2);
-	    else p = 0.0;
-	}
-	return p;
-    }
-    xs = h - 1.0;
-    d = x - h + 2.0/3.0 - .08/h;
-    xx = 2.0 * xs;
-    if (floateq(x, xs)) zz = - (1.0/3.0 + .08/h) / sqrt(xx);
-    else {
-	xh = x - xs;
-	absx = (xh>0.0)? xh : -xh;
-	aa = xs / x;
-	bb = xs * log(aa) + x - xs;
-	zz = d * sqrt(bb)/absx;
-    }
-    p = normal(zz);
-    return p;
+    if (x < 0) return 1.0;
+    else return 1.0 - chdtr(df, x);
 }
 
 /**
@@ -237,28 +125,19 @@ double chisq (const double x, const int df)
  *
  */
 
-double normal (const double x)
+double normal (double x)
 {
-    const double a1 = .0705230784;
-    const double a2 = .0422820123;
-    const double a3 = .0092705272;
-    const double a4 = .0001520143;
-    const double a5 = .0002765672;
-    const double a6 = .0000430638;
-    double absx, xx, zz, p;
+    return 1.0 - ndtr(x);
+}
 
-    absx = (x > 0.0)? x : -x;
-    if (absx <= 14.14) zz = .7071067812*absx;
-    else zz = 10.0;
-    xx = a6*zz + a5;
-    xx = xx*zz + a4;
-    xx = xx*zz + a3;
-    xx = xx*zz + a2;
-    xx = xx*zz + a1;
-    xx = xx*zz + 1.0;
-    p = 0.5 * pow(xx, -16.0);
-    if (x > 0.0) p = 1.0 - p;
-    return (1.0 - p);
+double normal_cdf (double x)
+{
+    return ndtr(x);
+}
+
+double normal_pdf (double x)
+{
+    return (1.0 / sqrt(2.0 * M_PI)) * exp(-0.5 * x * x);
 }
 
 /**
@@ -285,8 +164,9 @@ double batch_pvalue (const char *str,
 {
     int i, df1 = 0, df2 = 0;
     char stat = 0;
-    double xx = NADBL, mean = 0, variance = 0, xval = 0;
+    double xx = NADBL, mean = 0, variance = 0, xval = 0, tmp;
     char cmd[7], df1str[9], df2str[9], fstr[9]; 
+    int gotvar, err = 0;
 
     for (;;) {
 	if (sscanf(str, "%c,%[^,],%[^,],%s", &stat, df1str, df2str, fstr) == 4)
@@ -310,133 +190,176 @@ double batch_pvalue (const char *str,
 	break;
     }
 
-    if (isalpha((unsigned char) df1str[0])) {
+    if (isalpha((unsigned char) *df1str)) {
+	gotvar = 0;
 	for (i=0; i<pdinfo->v; i++) {
 	    if (strcmp(df1str, pdinfo->varname[i]) == 0) {
+		gotvar = 1;
 		df1 = (int) Z[i][0];
 		mean = Z[i][0];
 		break;
 	    }
 	}
+	if (!gotvar) {
+	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), df1str);
+	    err = 1;
+	}
     } else {
-	df1 = atoi(df1str);
-	mean = atof(df1str);
+	if (*df1str && check_atof(df1str)) {
+	    err = 1;
+	} else {
+	    df1 = atoi(df1str);
+	    mean = atof(df1str);
+	}
     }
 
-    if (isalpha((unsigned char) df2str[0])) {
+    if (isalpha((unsigned char) *df2str)) {
+	gotvar = 0;
 	for (i=0; i<pdinfo->v; i++) {
 	    if (strcmp(df2str, pdinfo->varname[i]) == 0) {
+		gotvar = 1;
 		df2 = (int) Z[i][0];
 		variance = Z[i][0];
 		break;
 	    }
 	}
+	if (!gotvar) {
+	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), df2str);
+	    err = 1;
+	}
     } else {
-	df2 = atoi(df2str);
-	variance = atof(df2str);
+	if (*df2str && check_atof(df2str)) {
+	    err = 1;
+	} else {
+	    df2 = atoi(df2str);
+	    variance = atof(df2str);
+	}
     }
 
-    if (isalpha((unsigned char) fstr[0])) {
+    if (isalpha((unsigned char) *fstr)) {
+	gotvar = 0;
 	for (i=0; i<pdinfo->v; i++) {
 	    if (strcmp(fstr, pdinfo->varname[i]) == 0) {
+		gotvar = 1;
 		xval = get_xvalue(i, Z, pdinfo);
 		if (na(xval)) {
-		    pprintf(prn, _("\nstatistic has missing value code\n"));
+		    pputs(prn, _("\nstatistic has missing value code\n"));
 		    return NADBL;
 		}		
 		break;
 	    }
 	}
-    } else xval = atof(fstr);
+	if (!gotvar) {
+	    sprintf(gretl_errmsg, _("Unknown variable '%s'"), fstr);
+	    err = 1;
+	}
+    } else {
+	if (*fstr && check_atof(fstr)) err = 1;
+	else xval = atof(fstr);
+    }
+
+    if (err) {
+	pprintf(prn, "%s\n", gretl_errmsg);
+	return NADBL;
+    }
 
     switch (stat) {
 
     case '1':
     case 'z':
     case 'n':
-	xx = normal(xval);
+	tmp = xval;
+	if (xval > 0.0) tmp = -tmp;
+	xx = normal_cdf(tmp);
 	if (xx < 0) {
-	    pprintf(prn, _("\np-value calculation failed\n"));
+	    pputs(prn, _("\np-value calculation failed\n"));
 	    return -1;
 	}	
 	pprintf(prn, _("\nStandard normal: area to the %s "
-		"of %f = %.4g\n"), (xval > 0)? _("right"): _("left"), 
+		"of %g = %g\n"), (xval > 0)? _("right"): _("left"), 
 		xval, xx);
-	pprintf(prn, _("(two-tailed value = %.4g; complement = %.4g)\n"), 
+	pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		2.0 * xx, 1.0 - 2.0 * xx);
 	return xx;
 
     case '2':
     case 't':
 	if (!*fstr || !*df1str) {
-	    pprintf(prn, _("\npvalue for t: missing parameter\n"));
+	    pputs(prn, _("\npvalue for t: missing parameter\n"));
 	    return -1;
 	}
 	xx = tprob(xval, df1);
 	if (xx < 0) {
-	    pprintf(prn, _("\np-value calculation failed\n"));
+	    pputs(prn, _("\np-value calculation failed\n"));
 	    return -1;
 	}
-	pprintf(prn, _("\nt(%d): area to the %s of %f = %.4g\n"), 
+	pprintf(prn, _("\nt(%d): area to the %s of %g = %g\n"), 
 		df1, (xval > 0)? _("right"): _("left"),
 		xval, 0.5 * xx);
-	pprintf(prn, _("(two-tailed value = %.4g; complement = %.4g)\n"), 
+	pprintf(prn, _("(two-tailed value = %g; complement = %g)\n"), 
 		xx, 1.0 - xx);
 	return xx;
+
     case '3':
     case 'c':
     case 'x':
     case 'X':
 	if (!*fstr || !*df1str) {
-	    pprintf(prn, _("\npvalue for chi-square: missing parameter\n"));
+	    pputs(prn, _("\npvalue for chi-square: missing parameter\n"));
 	    return -1;
 	}
 	xx = chisq(xval, df1);
 	if (xx < 0) {
-	    pprintf(prn, _("\np-value calculation failed\n"));
+	    pputs(prn, _("\np-value calculation failed\n"));
 	    return -1;
 	}
-	pprintf(prn, _("\nChi-square(%d): area to the right of %f = %.4g\n"), 
+	pprintf(prn, _("\nChi-square(%d): area to the right of %g = %g\n"), 
 		df1, xval, xx);
-	pprintf(prn, _("(to the left: %.4g)\n"), 1.0 - xx);
+	pprintf(prn, _("(to the left: %g)\n"), 1.0 - xx);
 	return xx;
 
     case '4':
     case 'f':
     case 'F':
 	if (!*fstr || !*df1str || !*df2str) {
-	    pprintf(prn, _("\npvalue for F: missing parameter\n"));
+	    pputs(prn, _("\npvalue for F: missing parameter\n"));
 	    return -1;
 	}
 	xx = fdist(xval, df1, df2);
 	if (xx < 0) {
-	    pprintf(prn, _("\np-value calculation failed\n"));
+	    pputs(prn, _("\np-value calculation failed\n"));
 	    return -1;
 	}
-	pprintf(prn, _("\nF(%d, %d): area to the right of %f = %.4g\n"), 
+	pprintf(prn, _("\nF(%d, %d): area to the right of %g = %g\n"), 
 		df1, df2, xval, xx);
-	pprintf(prn, _("(to the left: %.4g)\n"), 1.0 - xx);
+	pprintf(prn, _("(to the left: %g)\n"), 1.0 - xx);
 	return xx;
 
     case '5':
     case 'g':
     case 'G':
-	xx = _gammadist(mean, variance, xval, 2);
+	xx = gamma_dist(mean, variance, xval, 2);
 	if (na(xx))
-	    pprintf(prn, _("\nError computing gamma distribution\n"));
+	    pputs(prn, _("\nError computing gamma distribution\n"));
 	else
 	    pprintf(prn, _("\nGamma (mean %g, variance %g, shape %g, scale %g):"
-		    "\n area to the right of %f = %.4g\n"), 
+		    "\n area to the right of %g = %g\n"), 
 		    mean, variance, mean*mean/variance, variance/mean,
 		    xval, 1.0 - xx);
 	return xx;
 
     default:
-	pprintf(prn, _("\nunrecognized pvalue code\n"));
+	pputs(prn, _("\nunrecognized pvalue code\n"));
 	return NADBL;
     }
 }
 
+static void putxx (double xx)
+{
+    if (xx < 0.0001) puts("< 0.0001");
+    else printf("%g\n", xx);
+}
+
 /**
  * interact_pvalue:
  * 
@@ -469,19 +392,19 @@ void interact_pvalue (void)
 	    putchar('\n');
 	    return;
 	case 1:		
-	    _pnormal();
+	    pnormal();
 	    break;
 	case 2:		
-	    _ptvalue();
+	    ptvalue();
 	    break;
 	case 3:		
-	    _pchisq();
+	    pchisq();
 	    break;
 	case 4:		
-	    _pfvalue();
+	    pfvalue();
 	    break;
 	case 5:
-	    _pgamma();
+	    pgamma();
 	    break;
 	default:	
 	    puts(_("\ninvalid choice"));
@@ -497,105 +420,105 @@ void interact_pvalue (void)
 
 /* ........................................................ */
 
-static void _pnormal (void)
+static void pnormal (void)
 {
     double xx, zx; 
 
-    printf("%s", negval);
-    zx = _getvalue();
-    if(zx < 0.0) return;
+    printf("%s", _(negval));
+    zx = getx();
+    if (zx < 0.0) return;
     xx = normal(zx);
     printf(_("\nFor the standard normal, area (one-tail) to the "
 	   "right of %g is "), zx);
-    _putxx(xx);
+    putxx(xx);
 }
 
 /* ........................................................ */
 
-static void _ptvalue (void)
+static void ptvalue (void)
 {
     int n;
     double xx, zx, xsq; 
 
-    _enterdf(" ");
-    n = (int) _getvalue();
+    getdf(" ");
+    n = (int) getx();
     if (n <= 0) return;
-    printf("%s", negval);
-    zx = _getvalue();
+    printf("%s", _(negval));
+    zx = getx();
     if(zx < 0.0) return;
     xsq = zx * zx;
     xx = fdist(xsq, 1, n)/2.0;
     printf(_("\nFor Student's t(%d), area (one-tail) to the "
 	   "right of %g is "), n, zx);
-    _putxx(xx);
+    putxx(xx);
 }
 
 /* ........................................................ */
 
-static void _pchisq (void)
+static void pchisq (void)
 {
     int n;
     double xx, zx; 
 
-    _enterdf(" ");
-    n = (int) _getvalue();
+    getdf(" ");
+    n = (int) getx();
     if (n <= 0) return;
-    printf("%s", negval);
-    zx = _getvalue();
+    printf("%s", _(negval));
+    zx = getx();
     if(zx < 0.0) return;
     xx = chisq(zx, n);
     printf(_("\nFor Chi-square(%d), area to the right of %g is "), 
 	   n, zx);
-    _putxx(xx);
+    putxx(xx);
 }
 
 /* ........................................................ */
 
-static void _pfvalue (void)
+static void pfvalue (void)
 {
     int m, n;
     double xx, zx; 
 
-    _enterdf(_(" for the numerator "));
-    m = (int) _getvalue();
+    getdf(_(" for the numerator "));
+    m = (int) getx();
     if (m <= 0) return;
-    _enterdf(_(" for the denominator "));
-    n = (int) _getvalue();
+    getdf(_(" for the denominator "));
+    n = (int) getx();
     if (n <= 0) return;
-    printf("%s", negval);
-    zx = _getvalue();
+    printf("%s", _(negval));
+    zx = getx();
     if (zx < 0.0) return;
     xx = fdist(zx, m, n);
     printf(_("\nFor F(%d, %d), area to the right of %g is "),
 	   m, n, zx);
-    _putxx(xx);
+    putxx(xx);
 }
 
 /* ........................................................ */
 
-static void _pgamma (void) 
+static void pgamma (void) 
 {
     double mean, variance;
     double xx, zx; 
 
     printf(_("\nEnter the mean: "));
-    mean = _getvalue();
+    mean = getx();
     if (mean <= 0) return;
     printf(_("\nEnter the variance: "));
-    variance = _getvalue();
+    variance = getx();
     if (variance <= 0) return;
-    printf("%s", negval);
-    zx = _getvalue();
+    printf("%s", _(negval));
+    zx = getx();
     if (zx < 0.0) return;
-    xx = 1.0 - _gammadist(mean, variance, zx, 2);
+    xx = 1.0 - gamma_dist(mean, variance, zx, 2);
     printf(_("\nFor Gamma (mean %g, variance %g), area to the right of %g is "),
 	   mean, variance, zx);
-    _putxx(xx);
+    putxx(xx);
 }
 
 /* ........................................................ */
 
-static double _getvalue (void)
+static double getx (void)
 {
     double aa;
 
@@ -607,7 +530,7 @@ static double _getvalue (void)
 
 /* ........................................................ */
 
-static void _enterdf (const char *str)
+static void getdf (const char *str)
 {
     printf(_("\nEnter d.f.%s(value <= 0 will exit menu): "), str);
 }
@@ -622,7 +545,7 @@ static void _enterdf (const char *str)
  *
  */
 
-double f_crit_a (const double a, const int df1, const int df2)
+double f_crit_a (double a, int df1, int df2)
 {
     double x = 1.0;
 
@@ -653,11 +576,10 @@ static int parse_critical_input (const char *str, int *i,
 
 /**
  * print_critical:
- * @str: the command line, which should be of one of the following forms:
+ * @line: the command line, which should be of one of the following forms:
  * critical t df (student's t)
  * critical X df (chi-square)
  * critical F dfn dfd (F distribution)
- * @ppaths: pointer to paths information struct.
  * @prn: gretl printing struct.
  *
  * Prints critical values for the specified distribution at the
@@ -667,9 +589,9 @@ static int parse_critical_input (const char *str, int *i,
  *
  */
 
-int print_critical (const char *line, PATHS *ppaths, PRN *prn)
+int print_critical (const char *line, PRN *prn)
 {
-    void *handle;
+    void *handle = NULL;
     void *funp = NULL;
     void (*norm_table)(PRN *, int) = NULL;
     void (*dw)(int, PRN *) = NULL;
@@ -677,49 +599,43 @@ int print_critical (const char *line, PATHS *ppaths, PRN *prn)
     void (*chicrit)(int, PRN *, int) = NULL;
     int i, n = -1, df = -1, err = 0;
 
-    if (open_plugin(ppaths, "stats_tables", &handle)) return 1;
-
     if (parse_critical_input(line, &i, &df, &n)) {
-	pprintf(prn, _("Invalid input\n"));
+	pputs(prn, _("Invalid input\n"));
 	err = 1;
     }
 
     if ((0 < i && i < 4 && df <= 0) || (i == 3 && n <= 0)) {
-	pprintf(prn, _("Invalid degrees of freedom\n"));
+	pputs(prn, _("Invalid degrees of freedom\n"));
 	err = 1;
     }
     else if (i == 4 && n <= 0) {
-	pprintf(prn, _("Invalid sample size\n"));
+	pputs(prn, _("Invalid sample size\n"));
 	err = 1;
     }    
 
-    if (err) {
-	close_plugin(handle);
-	return 1;
-    }
+    if (err) return 1;
 
     switch (i) {
     case 0: /* normal */
-	funp = norm_table = get_plugin_function("norm_lookup", handle);
+	funp = norm_table = get_plugin_function("norm_lookup", &handle);
 	break;
     case 1: /* t */
-	funp = tcrit = get_plugin_function("t_lookup", handle);
+	funp = tcrit = get_plugin_function("t_lookup", &handle);
 	break;
     case 2: /* chi-square */
-	funp = chicrit = get_plugin_function("chisq_lookup", handle);
+	funp = chicrit = get_plugin_function("chisq_lookup", &handle);
 	break;
     case 3: /* F */
 	break;
     case 4: /* DW */
-	funp = dw = get_plugin_function("dw_lookup", handle);
+	funp = dw = get_plugin_function("dw_lookup", &handle);
 	break;
     default:
 	break;
     }
 
     if (i != 3 && funp == NULL)  {
-	pprintf(prn, _("Couldn't load plugin function\n"));
-	close_plugin(handle);
+	pputs(prn, _("Couldn't load plugin function\n"));
 	return 1;
     }
     
@@ -747,7 +663,9 @@ int print_critical (const char *line, PATHS *ppaths, PRN *prn)
 	break;
     }
 
-    close_plugin(handle);
+    if (handle != NULL) {
+	close_plugin(handle);
+    }
 
     return 0;
 }
@@ -763,8 +681,6 @@ static const double tolerance = 1e-7;
 
 /* internal functions */
 
-static double gamma_stirling (double x);
-static double gamma_12 (double x);
 static double gamma_integral (double lambda, double x);
 static double gamma_integral_expansion (double lambda, double x);
 static double gamma_integral_fraction (double lambda, double x);
@@ -772,27 +688,7 @@ static double gammadist_wilson_hilferty (double shape, double scale, double x);
 
 /* exported functions */
 
-double _gamma_func (double x)
-{
-    double xx;
-
-    if (x > 171) { 
-	return NADBL; 
-    }
-    if (x >= 6)
-	xx = gamma_stirling(x);
-    else if (x > 2)
-	xx = (x-1) * _gamma_func(x-1);
-    else if (x < 1)
-	xx = _gamma_func(x+1)/x;
-    else 
-	xx = gamma_12(x);
-    return xx;
-} 
-
-/* ........................................................ */
-
-double _gammadist (double s1, double s2, double x, int control)
+double gamma_dist (double s1, double s2, double x, int control)
      /* Control 1 : s1, s2 = shape, scale
                 2 : s1, s2 = expectation, variance
         Returns NADBL (-999.0) on error 
@@ -815,52 +711,13 @@ double _gammadist (double s1, double s2, double x, int control)
     else {
 	xx = gamma_integral(shape, x/scale);
 	if (na(xx)) return xx;
-	xx /= _gamma_func(shape);
+	xx /= cephes_gamma(shape);
     }
     return xx;
 }
 
 /* end exported functions */
 
-static double gamma_stirling (double x)
-     /* Stirling's series, only to be applied for large x.
-	Abramowitz and Stegun, p. 257 
-     */
-{
-    double x2, xx;
-
-    x2 = x*x;
-    xx = 1 + (double)(1)/12/x + (double)(1)/288/x2
-	- (double)(139)/51840/x/x2 
-	- (double)(571)/2488320/(x2*x2);
-    xx *= exp(-x + (x-(double)(1)/2) *log(x))*sqrt(2*M_PI);
-    return xx;
-}
-
-static double gamma_12 (double x)
-     /* Polynomial approximation, 6.1.36, 
-	Abramowitz and Stegun, p. 257
-	error < 3e-7. 
-     */
-{
-    const double b1 = -0.577191652;
-    const double b2 = 0.988205891;
-    const double b3 = -0.897056937;
-    const double b4 = 0.918206857;
-    const double b5 = -0.756704078;
-    const double b6 = 0.482199394;
-    const double b7 = -0.193527818;
-    const double b8 = 0.035868343;
-    double xx, x1, x2, x4;
-
-    x1 = x-1;
-    x2 = x1*x1;
-    x4 = x2*x2;
-    xx = 1 + b1*x1 + b2*x2 + b3*x1*x2 + b4*x4 + b5*x4*x1;
-    xx += b6*x4*x2 + b7*x4*x2*x1 + b8*x4*x4;
-    return xx;
-}
-
 static double gamma_integral_expansion (double lambda, double x)
      /* Expansion of Gamma Integral int_0^x t^lambda-1 exp(-t)
 	Abramowitz and Stegun p. 262
@@ -909,7 +766,7 @@ static double gamma_integral_fraction (double lambda, double x)
 	if (fabs(q2) > 0)  r2 = p2/q2;
     } while (!((fabs(r2-r1) < tolerance ) || (fabs(r2-r1) < tolerance * r2)
 	       || (c == 100)));
-    xx = _gamma_func(lambda);
+    xx = cephes_gamma(lambda);
     xx -= exp(-x + lambda * log(x)) * r2;
     if (c == 100) return NADBL;
     return xx;
