@@ -27,15 +27,20 @@ int tex_print_equation (const MODEL *pmod, const DATAINFO *pdinfo,
 int tex_print_model (const MODEL *pmod, const DATAINFO *pdinfo, 
 		     const int standalone, PRN *prn);
 
+int tex_print_coeff (const DATAINFO *pdinfo, const MODEL *pmod, 
+		      int c, PRN *prn);
+
 int tabprint (const MODEL *pmod, const DATAINFO *pdinfo,
 	      const PATHS *ppaths, char *texfile,
-	      const int model_count, int oflag);
+	      const int model_count, unsigned long oflag);
 
 int eqnprint (const MODEL *pmod, const DATAINFO *pdinfo,
 	      const PATHS *ppaths, char *texfile,
-	      const int model_count, int oflag);
+	      const int model_count, unsigned long oflag);
 
 char *tex_escape (char *targ, const char *src);
 
+void tex_dcolumn_double (double xx, char *numstr);
+
 
 
