@@ -17,9 +17,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifndef VERSION_H
-#define VERSION_H
+#ifndef GRETL_VERSION_H
+#define GRETL_VERSION_H
 
-const char *version_string = "0.992";
+#define LIBGRETL_CURRENT   0
+#define LIBGRETL_REVISION  20
+#define LIBGRETL_AGE       0
+
+const char *version_string = "1.2.7";
 
 #endif
