@@ -17,8 +17,10 @@
  *
  */
 
-#include "libgretl.h"
 #include <gtk/gtk.h>
+
+#include "libgretl.h"
+
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include "importer.h"
@@ -37,8 +39,6 @@ typedef enum {
     VALUE_ARRAY     = 80
 } ValueType;
 
-char *errbuf;
-
 #include "import_common.c"
 
 static void wsheet_init (wsheet *sheet)
@@ -48,6 +48,7 @@ static void wsheet_init (wsheet *sheet)
     sheet->Z = NULL;
     sheet->varname = NULL;
     sheet->label = NULL;
+    sheet->name = NULL;
 }
 
 static void wsheet_free (wsheet *sheet)
@@ -123,19 +124,19 @@ static int wsheet_allocate (wsheet *sheet, int cols, int rows)
 	sheet->Z[i] = malloc(rows * sizeof **(sheet->Z));
 	if (sheet->Z[i] == NULL) return 1;
 	for (t=0; t<rows; t++)
-	    sheet->Z[i][t] = -999.0;
+	    sheet->Z[i][t] = NADBL;
     }
 
     sheet->varname = malloc(cols * sizeof *(sheet->varname));
     for (i=0; i<cols; i++) {
-	sheet->varname[i] = malloc(9 * sizeof **(sheet->varname));
+	sheet->varname[i] = malloc(VNAMELEN * sizeof **(sheet->varname));
 	if (sheet->varname[i] == NULL) return 1;
 	sheet->varname[i][0] = '\0';
     }
 
     sheet->label = malloc(rows * sizeof *(sheet->label));
     for (t=0; t<rows; t++) {
-	sheet->label[t] = malloc(9 * sizeof **(sheet->label));
+	sheet->label[t] = malloc(VNAMELEN * sizeof **(sheet->label));
 	if (sheet->label[t] == NULL) return 1;
 	sheet->label[t][0] = '\0';
     }
@@ -143,7 +144,42 @@ static int wsheet_allocate (wsheet *sheet, int cols, int rows)
     return 0;
 }
 
-int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet)
+static int wsheet_get_real_size (xmlNodePtr node, wsheet *sheet)
+{
+    xmlNodePtr p = node->xmlChildrenNode;
+    int maxrow = 0, maxcol = 0;
+    char *tmp;
+
+    while (p) {
+	if (!xmlStrcmp(p->name, (UTF) "Cell")) {
+	    int i, j;
+
+	    tmp = xmlGetProp(p, (UTF) "Row");
+	    if (tmp) {
+		i = atoi(tmp);
+		free(tmp);
+		if (i > maxrow) maxrow = i;
+	    }
+	    tmp = xmlGetProp(p, (UTF) "Col");
+	    if (tmp) {
+		j = atoi(tmp);
+		free(tmp);
+		if (j > maxcol) maxcol = j;
+	    }
+	}
+	p = p->next;
+    }
+
+    fprintf(stderr, "wsheet_get_real_size: maxrow=%d, maxcol=%d\n",
+	    maxrow, maxcol);
+
+    sheet->maxcol = maxcol;
+    sheet->maxrow = maxrow;
+
+    return 0;
+}
+
+static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 {
     xmlNodePtr p = node->xmlChildrenNode;
     char *tmp;
@@ -174,7 +210,7 @@ int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet)
 	if (!xmlStrcmp(p->name, (UTF) "Cell")) {
 	    int i_real = 0, t_real = 0;
 
-	    x = -999.0;
+	    x = NADBL;
 	    i = 0; t = 0;
 
 	    tmp = xmlGetProp(p, (UTF) "Col");
@@ -194,24 +230,30 @@ int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet)
 		if (tmp) {
 		    vtype = atoi(tmp);
 		    free(tmp);
+		} else { /* a formula perhaps? */
+		    pprintf(prn, _("Couldn't get value for col %d, row %d.\n"
+				   "Maybe there's a formula in the sheet?"),
+			    i, t);
+		    err = 1;
 		}
 		/* check the top-left cell */
-		if (i_real == 0 && t_real == 0) {
+		if (!err && i_real == 0 && t_real == 0) {
 		    if (VTYPE_IS_NUMERIC(vtype)) {
-			sprintf(errbuf, "Expected to find a variable name");
+			pputs(prn, _("Expected to find a variable name"));
 			err = 1;
 		    }
 		}
-		else if (i_real >= 1 && t_real == 0 && 
+		else if (!err && i_real >= 1 && t_real == 0 && 
 			 !(vtype == VALUE_STRING)) {
 		    /* ought to be a varname here */
-		    sprintf(errbuf, "Expected to find a variable name");
+		    pputs(prn, _("Expected to find a variable name"));
 		    err = 1;
 		}
 		if (!err && (tmp = xmlNodeGetContent(p))) {
-		    if (VTYPE_IS_NUMERIC(vtype) ||vtype == VALUE_STRING) {
-			if (i_real == 0) 
-			    strncat(sheet->label[t_real], tmp, 8);
+		    if (VTYPE_IS_NUMERIC(vtype) || vtype == VALUE_STRING) {
+			if (i_real == 0) {
+			    strncat(sheet->label[t_real], tmp, OBSLEN - 1);
+			}
 		    }
 		    if (VTYPE_IS_NUMERIC(vtype)) {
 			x = atof(tmp);
@@ -219,8 +261,15 @@ int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet)
 			toprows[t_real] = leftcols[i_real] = 0;
 		    }
 		    else if (vtype == VALUE_STRING) {
-			if (t_real == 0)
-			    strncat(sheet->varname[i_real], tmp, 8);
+			if (t_real == 0) {
+			    strncat(sheet->varname[i_real], tmp, VNAMELEN - 1);
+			    if (i_real == 0 && !strcmp(tmp, "obs")) {
+				; /* keep going */
+			    } else if (check_varname(sheet->varname[i_real])) {
+				invalid_varname(prn);
+				err = 1;
+			    }
+			}
 			toprows[t_real] = leftcols[i_real] = 1;
 		    }
 		    free(tmp);
@@ -230,18 +279,22 @@ int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet)
 	p = p->next;
     }
 
-    for (i=0; i<cols; i++)
-	if (leftcols[i]) sheet->text_cols += 1;
-    for (t=0; t<rows; t++)
-	if (toprows[t]) sheet->text_rows += 1;
+    if (!err) {
+	for (i=0; i<cols; i++)
+	    if (leftcols[i]) sheet->text_cols += 1;
+	for (t=0; t<rows; t++)
+	    if (toprows[t]) sheet->text_rows += 1;
 
-    if (sheet->text_rows > 1) {
-	sprintf(errbuf, "Found an extraneous row of text");
-	err = 1;
-    }
-    if (sheet->text_cols > 1) {
-	sprintf(errbuf, "Found an extraneous column of text");
-	err = 1;
+	if (sheet->text_rows > 1) {
+	    pputs(prn, _("Found an extraneous row of text"));
+	    pputc(prn, '\n');
+	    err = 1;
+	}
+	if (sheet->text_cols > 1) {
+	    pputs(prn, _("Found an extraneous column of text"));
+	    pputc(prn, '\n');
+	    err = 1;
+	}
     }
 
     free(toprows);
@@ -250,7 +303,7 @@ int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet)
     return err;
 }
 
-static int wsheet_get_data (const char *fname, wsheet *sheet) 
+static int wsheet_get_data (const char *fname, wsheet *sheet, PRN *prn) 
 {
     xmlDocPtr doc;
     xmlNodePtr cur, sub;
@@ -262,28 +315,26 @@ static int wsheet_get_data (const char *fname, wsheet *sheet)
 
     doc = xmlParseFile(fname);
     if (doc == NULL) {
-	sprintf(errbuf, "xmlParseFile failed on %s", fname);
+	pprintf(prn, _("xmlParseFile failed on %s"), fname);
 	return 1;
     }
 
     cur = xmlDocGetRootElement(doc);
     if (cur == NULL) {
-        sprintf(errbuf, "%s: empty document", fname);
+        pprintf(prn, _("%s: empty document"), fname);
 	xmlFreeDoc(doc);
 	return 1;
     }
 
     if (xmlStrcmp(cur->name, (UTF) "Workbook")) {
-        sprintf(errbuf, "File of the wrong type, root node not Workbook");
+        pputs(prn, _("File of the wrong type, root node not Workbook"));
 	xmlFreeDoc(doc);
 	return 1;
     }
 
-    wsheet_init(sheet);
-
     /* Now walk the tree */
     cur = cur->xmlChildrenNode;
-    while (cur != NULL && !got_sheet) {
+    while (!err && cur != NULL && !got_sheet) {
 	if (!xmlStrcmp(cur->name, (UTF) "Sheets")) {
 	    int sheetcount = 0;
 
@@ -322,8 +373,8 @@ static int wsheet_get_data (const char *fname, wsheet *sheet)
 			}
 			else if (got_sheet &&
 				 !xmlStrcmp(snode->name, (UTF) "Cells")) {
-			    /* error handling needed */
-			    wsheet_parse_cells(snode, sheet);
+			    wsheet_get_real_size(snode, sheet);
+			    err = wsheet_parse_cells(snode, sheet, prn);
 			}
 			snode = snode->next;
 		    }
@@ -353,7 +404,7 @@ static int wbook_record_name (char *name, wbook *book)
     return 0;
 }
 
-static int wbook_get_info (const char *fname, wbook *book) 
+static int wbook_get_info (const char *fname, wbook *book, PRN *prn) 
 {
     xmlDocPtr doc;
     xmlNodePtr cur, sub;
@@ -367,19 +418,19 @@ static int wbook_get_info (const char *fname, wbook *book)
 
     doc = xmlParseFile(fname);
     if (doc == NULL) {
-	sprintf(errbuf, "xmlParseFile failed on %s", fname);
+	pprintf(prn, _("xmlParseFile failed on %s"), fname);
 	return 1;
     }
 
     cur = xmlDocGetRootElement(doc);
     if (cur == NULL) {
-        sprintf(errbuf, "%s: empty document", fname);
+        pprintf(prn, _("%s: empty document"), fname);
 	xmlFreeDoc(doc);
 	return 1;
     }
 
     if (xmlStrcmp(cur->name, (UTF) "Workbook")) {
-        sprintf(errbuf, "File of the wrong type, root node not Workbook");
+        pputs(prn, _("File of the wrong type, root node not Workbook"));
 	xmlFreeDoc(doc);
 	return 1;
     }
@@ -412,8 +463,7 @@ static int wbook_get_info (const char *fname, wbook *book)
     return err;
 }
 
-static 
-int wsheet_setup (wsheet *sheet, wbook *book, int n)
+static int wsheet_setup (wsheet *sheet, wbook *book, int n)
 {
     size_t len = strlen(book->sheetnames[n]) + 1;
     
@@ -443,50 +493,97 @@ static int wsheet_labels_complete (wsheet *sheet)
 static int consistent_date_labels (wsheet *sheet)
 {
     int t, rows = sheet->maxrow + 1 - sheet->row_offset;
+    int tstart = 1 + sheet->row_offset;
     int pd = 0, pdbak = 0;
     double x, xbak = 0.0;
-    
-    for (t=1; t<rows; t++) {
-	if (sheet->label[t][0] == '\0') return 0;
-	pd = label_is_date(sheet->label[t]);
-	if (pd == 0) return 0;
-	x = atof(sheet->label[t]);
-	if (t == 1) pdbak = pd;
-	else { /* t > 1 */
-	    if (pd != pdbak) return 0;
+    char *test;
+
+    fputs("testing for consistent date labels\n", stderr);
+
+    for (t=tstart; t<rows; t++) {
+	test = sheet->label[t];
+	if (*test == '\0') {
+	    fprintf(stderr, " no: blank cell at row %d\n", t);
+	    return 0;
+	}
+	if (*test == '"' || *test == '\'') test++;
+	pd = label_is_date(test);
+	if (pd == 0) {
+	    fprintf(stderr, " no: label '%s' at row %d is not a date\n", 
+		    test, t);
+	    return 0;
+	}
+	x = atof(test);
+	if (t == tstart) {
+	    pdbak = pd;
+	} else { 
+	    if (pd != pdbak) {
+		fprintf(stderr, " no: got inconsistent data frequencies %d and %d\n",
+			pdbak, pd);
+		return 0;
+	    }		
 	    if (x <= xbak) return 0;
 	}
 	xbak = x;
     }
+
     return pd;
 }
 
+static int rigorous_dates_check (wsheet *sheet, DATAINFO *pdinfo)
+{
+    int t, rows = sheet->maxrow + 1 - sheet->row_offset;
+    int tstart = 1 + sheet->row_offset;
+    int n, nbak = 0, err = 0;
+
+    fputs("Doing rigorous dates check\n", stderr);
+
+    for (t=tstart; t<rows; t++) {
+	n = dateton(sheet->label[t], pdinfo);
+	if (t > tstart && n != nbak + 1) {
+	    fprintf(stderr, "problem: date[%d]='%s' but date[%d]='%s'\n",
+		    t - tstart + 1, sheet->label[t],
+		    t - tstart, sheet->label[t-1]);
+	    err = 1;
+	    break;
+	}
+	nbak = n;
+    }    
+
+    return err;
+}
+
 int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
-		    char *errtext)
+		    PRN *prn)
 {
     wbook book;
     wsheet sheet;
     int err = 0, sheetnum = -1;
-    double **newZ;
+    double **newZ = NULL;
     DATAINFO *newinfo;
 
-    errbuf = errtext;
-    errbuf[0] = '\0';
-
     newinfo = datainfo_new();
     if (newinfo == NULL) {
-	sprintf(errtext, _("Out of memory\n"));
+	pputs(prn, _("Out of memory\n"));
 	return 1;
     }
 
-    if (wbook_get_info(fname, &book)) {
-	sprintf(errbuf, "Failed to get workbook info");
-	err = 1;
-    } else
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "C");
+#endif
+
+    if (wbook_get_info(fname, &book, prn)) {
+	pputs(prn, _("Failed to get workbook info"));
+#ifdef ENABLE_NLS
+	setlocale(LC_NUMERIC, "");
+#endif
+	return 1;
+    } else {
 	wbook_print_info(&book);
+    }
 
     if (book.nsheets == 0) {
-	sprintf(errbuf, "No worksheets found");
+	pputs(prn, _("No worksheets found"));
     }
     else if (book.nsheets > 1) {
 	wsheet_menu(&book, 1);
@@ -497,34 +594,38 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	sheetnum = 0;
     }
 
-    if (sheetnum >= 0) {
+    if (book.selected == -1) err = -1;
+
+    wsheet_init(&sheet);
+
+    if (!err && sheetnum >= 0) {
 	fprintf(stderr, "Getting data...\n");
 	if (wsheet_setup(&sheet, &book, sheetnum)) {
-	    sprintf(errbuf, "error in wsheet_setup()");
+	    pputs(prn, _("error in wsheet_setup()"));
 	    err = 1;
 	} else {
-	    err = wsheet_get_data(fname, &sheet);
-	    if (!err) 
+	    err = wsheet_get_data(fname, &sheet, prn);
+	    if (!err) {
 		wsheet_print_info(&sheet);
+	    }
 	}
-    } /* else?? */
+    } /* else ??? */
 
     wbook_free(&book);
 
     if (!err) {
-	int i, t, i_sheet, label_strings = sheet.text_cols;
+	int i, j, t, i_sheet, label_strings = sheet.text_cols;
 	int time_series = 0;
+	int blank_cols = 0;
 
-	if (sheet.text_cols == 0 && obs_column(sheet.label[0])) {
+	if (obs_column_heading(sheet.label[0])) {
 	    int pd = consistent_date_labels(&sheet);
 
 	    if (pd) {
-		newinfo->pd = pd;
-		newinfo->sd0 = atof(sheet.label[1]);
-		strcpy(newinfo->stobs, sheet.label[1]);
-		newinfo->time_series = TIME_SERIES;
-		sheet.text_cols = 1;
-		time_series = 1;
+		time_series_setup(sheet.label[1], newinfo, pd,
+				  &sheet.text_cols,
+				  &time_series, &label_strings);
+		rigorous_dates_check(&sheet, newinfo);
 	    }
 	}
 
@@ -542,18 +643,30 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	    newinfo->pd = 1;
 	    newinfo->time_series = 0;
 	} else {
-	    ntodate(newinfo->endobs, newinfo->n - 1, newinfo);
+	    ntodate_full(newinfo->endobs, newinfo->n - 1, newinfo);
+	    fprintf(stderr, "endobs='%s'\n", newinfo->endobs);
 	}
-	newinfo->extra = 0; 
 
+	j = 1;
 	for (i=1; i<newinfo->v; i++) {
 	    i_sheet = i - 1 + sheet.text_cols;
-	    strcpy(newinfo->varname[i], sheet.varname[i_sheet]);
-	    for (t=0; t<newinfo->n; t++) {
-		newZ[i][t] = sheet.Z[i_sheet][t+1];
+	    if (*sheet.varname[i_sheet] == '\0') {
+		blank_cols++;
+	    } else {
+		strcpy(newinfo->varname[j], sheet.varname[i_sheet]);
+		for (t=0; t<newinfo->n; t++) {
+		    newZ[j][t] = sheet.Z[i_sheet][t+1];
+		}
+		j++;
 	    }
 	}
 
+	if (blank_cols > 0) {
+	    fprintf(stderr, "Dropping %d apparently blank column(s)\n", 
+		    blank_cols);
+	    dataset_drop_vars(blank_cols, &newZ, newinfo);
+	}
+
 	if (label_strings && wsheet_labels_complete(&sheet)) {
 	    char **S = NULL;
 
@@ -569,16 +682,15 @@ int wbook_get_data (const char *fname, double ***pZ, DATAINFO *pdinfo,
 	    *pZ = newZ;
 	    *pdinfo = *newinfo;
 	} else {
-	    PRN prn;
-
-	    prn.fp = NULL;
-	    prn.buf = errtext;
-	    err = merge_data(pZ, pdinfo, newZ, newinfo, &prn, 1);
+	    err = merge_data(pZ, pdinfo, newZ, newinfo, prn);
 	}
     } 	    
 
     wsheet_free(&sheet);
 
-    return err;
+#ifdef ENABLE_NLS
+    setlocale(LC_NUMERIC, "");
+#endif
 
+    return err;
 }
